%=================================================================
% Tutorial to 'Numerical Methods for PDEs', WS 01/02
% Lecture given by Dr.C.Coclici, assisted by M.Rheinlnder
%=================================================================
% Check numerically the order of convergence for the
%   Upwind scheme
%   Lax-Friedrichs scheme
%   Lax-Wendroff scheme.
%=================================================================
clear all;

alfa = -1;
beta =  1;
M   = 100;
a   = 0.5;
cfl = 0.7;
%rf1 %refinement factor
%rf2 %refinement factor

fhandle = @peak;
%fhandle = @jump;


method = input('Method (UP,LF,LW): ','s');
rf1    = input('1st refinement factor (non-negative integer): ');
rf2    = input('2nd refinement factor (non-negative integer): ');
if(rf1 > rf2) temp = rf1; rf1=rf2; rf2=temp; end;	

M1  = 2^rf1 * M;
M2  = 2^rf2 * M;
dx1 = (beta-alfa)/M1;
dx2 = (beta-alfa)/M2;
dt1 = dx1*cfl/a;
dt2 = dx2*cfl/a;
X1  = [alfa+dx1/2:dx1:beta-dx1/2];
X2  = [alfa+dx2/2:dx2:beta-dx2/2];
U1  = zeros(1,M1);
U2  = zeros(1,M2);
E1  = zeros(1,M1);
E2  = zeros(1,M2);
q   = 2^(rf2-rf1);
%ERR1
%ERR2

%---------------------------------
% Initialization
%---------------------------------
   t1 = 0; t2 = 0;
   IT1= 0; IT2= 0;
   P1 = init_P(method,M1,a*dt1/dx1);
   P2 = init_P(method,M2,a*dt2/dx2);
   U1 = feval(fhandle,alfa,beta,X1);
   U2 = feval(fhandle,alfa,beta,X2);

%---------------------------------
% Loop
%---------------------------------

while(1)
  figure(1); clf; hold on;
    Y2   = alfa + mod((X2-alfa) - a*t2, beta-alfa);
    UE = feval(fhandle,alfa,beta,Y2);
    plot([alfa-dx1/2 X1 beta+dx1/2], [U1(M1) U1  U1(1)],'g-');
    plot([alfa-dx2/2 X2 beta+dx2/2], [U2(M2) U2  U2(1)],'b-');
    plot([alfa-dx2/2 X2 beta+dx2/2], [UE(M2) UE  UE(1)],'y-');
    title('Comparison of different Refinements');
    xlabel('x'); ylabel('u');
    text(alfa+0.03*(beta-alfa),1.45,['IT1=' int2str(IT1)  ' IT2=' int2str(IT2)]);
    legend('coarse grid','fine grid','exact');
    axis([alfa beta -0.5 1.5]);
  figure(2); clf; hold on;
    title('Local Error: exact-numeric')
    xlabel('x'); ylabel('error');
    plot([alfa-dx1/2 X1 beta+dx1/2], [E1(M1) E1  E1(1)],'g-');
    plot([alfa-dx2/2 X2 beta+dx2/2], [E2(M2) E2  E2(1)],'b-');

  n = input('number of performing time-steps ','s');
  if(isempty(n)==1) n=1;
  elseif(n=='q') break; end;
  n = numeric(n);

  it1 =   n;
  it2 = q*n;

  for i=1:it1,
    t1     = t1 + dt1;
    U1     = (P1 * U1')';
    Y1     = alfa + mod((X1-alfa) - a*t1, beta-alfa);
    E1     = feval(fhandle,alfa,beta,Y1) - U1;
    E1_1(IT1+i) = norm(E1,1)*dx1;
    %E2_1(IT1+i) = norm(E1,2)*dx1;
  end;
  for i=1:it2,
    t2    = t2 + dt2;
    U2    = (P2 * U2')';
    Y2   = alfa + mod((X2-alfa) - a*t2, beta-alfa);
    E2   = feval(fhandle,alfa,beta,Y2) - U2;
    E1_2(IT2+i) = norm(E2,1)*dx2;
    %E2_2(IT2+i) = norm(E2,2)*dx2;
  end;
  IT1 = IT1 + it1;
  IT2 = IT2 + it2;
  r1 = (log(E1_1(IT1)) - log(E1_2(IT2)))/(log(dx1) - log(dx2));
  %r2 = (log(E2_1(IT1)) - log(E2_2(IT2)))/(log(dx1) - log(dx2));
  fprintf('convergence order: %f\n',r1);
end; %while

figure(3); clf; hold on;
  title('Time Behavior of L1 Error')
  xlabel('time'); ylabel('error');
  plot([dt1:dt1:IT1*dt1],E1_1,'g-');
  plot([dt2:dt2:IT2*dt2],E1_2,'b-');
  %plot([dt1:dt1:IT1*dt1],E2_1,'g:');
  %plot([dt2:dt2:IT2*dt2],E2_2,'b:');
  legend('coarse grid','fine grid');

