function [P] = init_P(method,M,c)

%------------------------------
% Upwind iteration matrix
%--------------------------------
if(method=='UP')
  if(c>=0)
    P(1,M) = c;
    P(1,1) = 1-c;
    for i=2:M-1,
      P(i,i-1) = c;
      P(i,i)   = 1-c;
    end;
    P(M,M-1) = c;
    P(M,M)   = 1-c;
  else
    P(1,1) = 1+c;
    P(1,2) = -c;
    for i=2:M-1,
      P(i,i)   = 1+c;
      P(i,i+1) =  -c;
    end;
    P(M,M)   = 1+c;
    P(M,M+1) =  -c;
  end;
%--------------------------------
% Lax-Friedrichs iteration matrix
%--------------------------------
elseif(method=='LF')
  P(1,M) = 0.5*(1+c);
  P(1,2) = 0.5*(1-c);
  for i=2:M-1,
    P(i,i-1) = 0.5*(1+c);
    P(i,i+1) = 0.5*(1-c);
  end;
  P(M,M-1) = 0.5*(1+c);
  P(M,1)   = 0.5*(1-c);
%--------------------------------
% Lax-Wendroff iteration matrix
%--------------------------------
elseif(method=='LW')
  P(1,M) = 0.5*(c+c^2);
  P(1,1) = 1-c^2;
  P(1,2) = 0.5*(c^2-c);
  for i=2:M-1,
    P(i,i-1) = 0.5*(c+c^2);
    P(i,i)   = 1-c^2;
    P(i,i+1) = 0.5*(c^2-c);
  end;
  P(M,M-1) = 0.5*(c+c^2);
  P(M,M)   = 1-c^2;
  P(M,1) = 0.5*(c^2-c);
end;


